==> tecfy_corev2_package-1.1.91.tar.gz: lib/views/components/entity/list_form.property.dart <==
part of tecfy_corev2_client_package;

class ListFormProperty extends StatefulWidget {
  final String entityId;
  final String? label;
  final String? description;
  final bool isRequired;
  final bool readOnly;
  final bool isAddNew;
  final List<dynamic> data;
  final Map<String, dynamic>? rootData;
  final Entity? rootEntity;
  final void Function(List<dynamic>)? onChanged;
  ListFormProperty(this.isAddNew,
      {required this.entityId,
      this.label,
      this.description,
      this.isRequired = false,
      this.data = const [],
      this.rootData,
      this.rootEntity,
      this.onChanged,
      Key? key,
      this.readOnly = false})
      : super(key: key);
  @override
  _ListFormPropertyState createState() => _ListFormPropertyState();
}

class _ListFormPropertyState extends TecfyState<ListFormProperty> {
  Entity? _entity;
  late ScrollController _scrollController;

  Future _onValueChanging(value, propName, Map<String, dynamic> data) async {
    _entity?.updatedProps = {};
    if (await _onValueChanged(value, propName, data)) {
      setState(() {});
    }
  }

  Future<bool> _onValueChanged(
      value, propName, Map<String, dynamic> data) async {
    if (_entity?.updatedProps[propName] != null) return false;
    _entity?.updatedProps[propName] = true;
    data[propName] = value;

    bool updated = await _entity!.setDefaultValues(data);

    // for (Property p in entity.editProperties) {
    //   var r = await p.isVisble(data, entity);
    //   if (r != p.isVisable) {
    //     p.isVisable = r;
    updated = true;
    //   }
    // }

    // check the calced properties
    for (PropertyNumber p in _entity?.calcProps ?? []) {
      if (_entity?.updatedProps[p.dbName] != null) continue;
      _entity?.updatedProps[p.dbName] = true;

      // if (p.calcedProps.contains(p.upperFirst(propName)) ||
      //     p.calcedProps.contains(propName)) {
      var vars = p.calcedProps
          .map((pro) => MapService.getValue(data, p.lowerFirst(pro)!) ?? 0)
          .toList();
      MapService.updateValue(data, p.dbName, p.getCalcResult(vars));
      //setState(() {});
      // }
    }

    // if (updated ||
    //     _entity!.editProperties.any((p) => p.filterValueField == propName))
    //   setState(() {});

    widget.onChanged?.call(widget.data);
    return updated;
  }

  _onUpdateData(Map<String, dynamic> data, Property property) {
    // we need this for Ramy's application to update the records when select an Item
    if (property.propertyType == PropertyTypes.Dropdown) {
      var prop = property as PropertyDropdown;
      if (MapService.getValue(data, prop.dbName) == null) return;
      var props = property.overwritePropertiesOnSelect?.keys ?? [];
      for (var key in props) {
        _onValueChanged(
            MapService.getValue(data, '${prop.dbName}.$key'), //TODO: fix dots
            property.overwritePropertiesOnSelect![key],
            data);
      }
      setState(() {});
    }
  }

  _scanBarcode() async {
    var result =
        await AppNavigator.showAppDialog(context, (c) => ScannerCameraDialog());

    if (result != null) {
      if (kDebugMode)
        print(
            '-------------start get data for ($result) for [${_entity!.dbName}]');
      var service = TecfyCoreApp.entityServices[_entity!.dbName.toLowerCase()];
      var data = await service?.getDataByBarcode(context, result, {});

      if (data?.data != null) setState(() => widget.data.add(data!.data));
    }
  }

  @override
  void initState() {
    _scrollController = ScrollController();
    _entity = EntityService.getEntity(widget.entityId)!;
    if (_entity == null)
      print(
          'XXXXXXXXXXXX  Error Cannot find entity structure  ${widget.entityId}');
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutService.bindWidget(context,
        mobileWidget: _getMobile,
        descktopWidget: _getDesktop,
        tabletWidget: _getMobile);
  }

  Widget _getDesktop() {
    return Tooltip(
      message: widget.description ?? '',
      child: Column(children: [
        Scrollbar(
            controller: _scrollController,
            child: SingleChildScrollView(
              controller: _scrollController,
              scrollDirection: Axis.horizontal,
              child: DataTable(
                showCheckboxColumn: true,
                dataRowMaxHeight: double.infinity,
                columns: _entity!.editProperties
                    .map<DataColumn>((p) => DataColumn(
                        label: TecfyText(p.name),
                        numeric: p.valueType == ValueTypes.Number,
                        tooltip: p.nameEn))
                    .toList()
                  ..add(DataColumn(label: TecfyText(''))), // for remove button
                rows: widget.data
                    .map<DataRow>(
                      (dataRow) => DataRow(
                        key: ObjectKey(dataRow),
                        cells: _entity!.editProperties
                            .map<DataCell>(
                              (p) => DataCell(
                                SizedBox(
                                  width: 170,
                                  child: PorpertyComponent(
                                    readOnly: widget.readOnly,
                                    // noLabel: true,
                                    isAddNew: widget.isAddNew,
                                    property: p,
                                    data: dataRow,
                                    rootData: widget.rootData,
                                    rootEntity: widget.rootEntity,
                                    entity: _entity!,
                                    onChange: (v) {
                                      _onValueChanging(v, p.dbName, dataRow);
                                    },
                                    onUpdateData: (v) => _onUpdateData(v, p),
                                  ),
                                ),
                              ),
                            )
                            .toList()
                          ..add(
                            DataCell(
                              widget.readOnly
                                  ? SizedBox()
                                  : IconButton(
                                      icon: Icon(Icons.close),
                                      tooltip: trans('Remove'),
                                      onPressed: widget.readOnly
                                          ? null
                                          : () => setState(() {
                                                widget.data.remove(dataRow);
                                                _getDesktop();
                                              })),
                            ),
                          ),
                      ),
                    )
                    .toList(),
              ),
            )),
        SizedBox(height: 30),
        widget.readOnly
            ? SizedBox.shrink()
            : SizedBox(
                width: 250,
                child: AppButtonIconText(
                  label: '${trans('Click to add new')} ${widget.label}',
                  icon: Icon(Icons.add, color: Colors.white),
                  onPressed: _addNewRow,
                ),
              ),
      ]),
    );
  }

  Widget _getMobile() {
    return Tooltip(
      message: widget.description,
      child: Column(
        children: widget.data
            .map<Widget>(
              (item) => InLineForm(
                widget.entityId,
                widget.isAddNew,
                readOnly: widget.readOnly,
                entity: _entity,
                data: item,
                onChanged: (v) => widget.onChanged?.call(widget.data),
                onDismiss: (v) => setState(() => widget.data.remove(v)),
              ),
            )
            .toList()
          ..add(
            widget.readOnly
                ? Container()
                : Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                    Container(
                      constraints: BoxConstraints(maxWidth: 150),
                      child: AppButtonIconText(
                        isFullRow: true,
                        label: '${trans('Click to add new')} ${widget.label}',
                        icon: Icon(Icons.add, color: Colors.white),
                        onPressed: _addNewRow,
                      ),
                    ),
                    SizedBox(width: 10),
                    _entity!.hasBarcode
                        ? IconButton(
                            icon: Icon(Icons.qr_code), onPressed: _scanBarcode)
                        : Container()
                  ]),
          ),
      ),
    );
  }

  Future _addNewRow() async {
    var row = Map<String, dynamic>();
    row['i'] = widget.data.length;
    setState(() => widget.data.add(row));
  }
}

==> tecfy_corev2_package-1.1.108.tar.gz: lib/views/components/entity/list_form.property.dart <==
part of tecfy_corev2_client_package;

class ListFormProperty extends StatefulWidget {
  final String entityId;
  final String? label;
  final String? description;
  final bool isRequired;
  final bool readOnly;
  final bool isAddNew;
  final List<dynamic> data;
  final Map<String, dynamic>? rootData;
  final Entity? rootEntity;
  final void Function(List<dynamic>)? onChanged;
  ListFormProperty(this.isAddNew,
      {required this.entityId,
      this.label,
      this.description,
      this.isRequired = false,
      this.data = const [],
      this.rootData,
      this.rootEntity,
      this.onChanged,
      Key? key,
      this.readOnly = false})
      : super(key: key);
  @override
  _ListFormPropertyState createState() => _ListFormPropertyState();
}

class _ListFormPropertyState extends TecfyState<ListFormProperty> {
  Entity? _entity;
  late ScrollController _scrollController;

  Future _onValueChanging(value, propName, Map<String, dynamic> data) async {
    _entity?.updatedProps = {};
    if (await _onValueChanged(value, propName, data)) {
      setState(() {});
    }
  }

  Future<bool> _onValueChanged(
      value, propName, Map<String, dynamic> data) async {
    if (_entity?.updatedProps[propName] != null) return false;
    _entity?.updatedProps[propName] = true;
    data[propName] = value;

    bool updated = await _entity!.setDefaultValues(data);

    // for (Property p in entity.editProperties) {
    //   var r = await p.isVisble(data, entity);
    //   if (r != p.isVisable) {
    //     p.isVisable = r;
    updated = true;
    //   }
    // }

    // check the calced properties
    for (PropertyNumber p in _entity?.calcProps ?? []) {
      if (_entity?.updatedProps[p.dbName] != null) continue;
      _entity?.updatedProps[p.dbName] = true;

      // if (p.calcedProps.contains(p.upperFirst(propName)) ||
      //     p.calcedProps.contains(propName)) {
      var vars = p.calcedProps
          .map((pro) => MapService.getValue(data, p.lowerFirst(pro)!) ?? 0)
          .toList();
      MapService.updateValue(data, p.dbName, p.getCalcResult(vars));
      //setState(() {});
      // }
    }

    // if (updated ||
    //     _entity!.editProperties.any((p) => p.filterValueField == propName))
    //   setState(() {});

    widget.onChanged?.call(widget.data);
    return updated;
  }

  _onUpdateData(Map<String, dynamic> data, Property property) {
    // we need this for Ramy's application to update the records when select an Item
    if (property.propertyType == PropertyTypes.Dropdown) {
      var prop = property as PropertyDropdown;
      if (MapService.getValue(data, prop.dbName) == null) return;
      var props = property.overwritePropertiesOnSelect?.keys ?? [];
      for (var key in props) {
        _onValueChanged(
            MapService.getValue(data, '${prop.dbName}.$key'), //TODO: fix dots
            property.overwritePropertiesOnSelect![key],
            data);
      }
      setState(() {});
    }
  }

  _scanBarcode() async {
    var result =
        await AppNavigator.showAppDialog(context, (c) => ScannerCameraDialog());

    if (result != null) {
      if (kDebugMode)
        print(
            '-------------start get data for ($result) for [${_entity!.dbName}]');
      var service = TecfyCoreApp.entityServices[_entity!.dbName.toLowerCase()];
      var data = await service?.getDataByBarcode(context, result, {});

      if (data?.data != null) setState(() => widget.data.add(data!.data));
    }
  }

  @override
  void initState() {
    _scrollController = ScrollController();
    _entity = EntityService.getEntity(widget.entityId);
    if (_entity == null)
      print(
          'XXXXXXXXXXXX  Error Cannot find entity structure  ${widget.entityId}');
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return LayoutService.bindWidget(context,
        mobileWidget: _getMobile,
        descktopWidget: _getDesktop,
        tabletWidget: _getMobile);
  }

  Widget _getDesktop() {
    return Tooltip(
      message: widget.description ?? '',
      child: Column(children: [
        Scrollbar(
            controller: _scrollController,
            child: SingleChildScrollView(
              controller: _scrollController,
              scrollDirection: Axis.horizontal,
              child: DataTable(
                showCheckboxColumn: true,
                dataRowMaxHeight: double.infinity,
                columns: _entity?.editProperties
                        .map<DataColumn>((p) => DataColumn(
                            label: TecfyText(p.name),
                            numeric: p.valueType == ValueTypes.Number,
                            tooltip: p.nameEn))
                        .toList() ??
                    <DataColumn>[]
                  ..add(DataColumn(label: TecfyText(''))), // for remove button
                rows: widget.data
                    .map<DataRow>(
                      (dataRow) => DataRow(
                        key: ObjectKey(dataRow),
                        cells: _entity?.editProperties
                                .map<DataCell>(
                                  (p) => DataCell(
                                    SizedBox(
                                      width: 170,
                                      child: PorpertyComponent(
                                        readOnly: widget.readOnly,
                                        // noLabel: true,
                                        isAddNew: widget.isAddNew,
                                        property: p,
                                        data: dataRow,
                                        rootData: widget.rootData,
                                        rootEntity: widget.rootEntity,
                                        entity: _entity!,
                                        onChange: (v) {
                                          _onValueChanging(
                                              v, p.dbName, dataRow);
                                        },
                                        onUpdateData: (v) =>
                                            _onUpdateData(v, p),
                                      ),
                                    ),
                                  ),
                                )
                                .toList() ??
                            <DataCell>[]
                          ..add(
                            DataCell(
                              widget.readOnly
                                  ? SizedBox()
                                  : IconButton(
                                      icon: Icon(Icons.close),
                                      tooltip: trans('Remove'),
                                      onPressed: widget.readOnly
                                          ? null
                                          : () => setState(() {
                                                widget.data.remove(dataRow);
                                                _getDesktop();
                                              })),
                            ),
                          ),
                      ),
                    )
                    .toList(),
              ),
            )),
        SizedBox(height: 30),
        widget.readOnly
            ? SizedBox.shrink()
            : SizedBox(
                width: 250,
                child: AppButtonIconText(
                  label: '${trans('Click to add new')} ${widget.label}',
                  icon: Icon(Icons.add, color: Colors.white),
                  onPressed: _addNewRow,
                ),
              ),
      ]),
    );
  }

  Widget _getMobile() {
    return Tooltip(
      message: widget.description,
      child: Column(
        children: widget.data
            .map<Widget>(
              (item) => InLineForm(
                widget.entityId,
                widget.isAddNew,
                readOnly: widget.readOnly,
                entity: _entity,
                data: item,
                onChanged: (v) => widget.onChanged?.call(widget.data),
                onDismiss: (v) => setState(() => widget.data.remove(v)),
              ),
            )
            .toList()
          ..add(
            widget.readOnly
                ? Container()
                : Row(mainAxisAlignment: MainAxisAlignment.center, children: [
                    Container(
                      constraints: BoxConstraints(maxWidth: 150),
                      child: AppButtonIconText(
                        isFullRow: true,
                        label: '${trans('Click to add new')} ${widget.label}',
                        icon: Icon(Icons.add, color: Colors.white),
                        onPressed: _addNewRow,
                      ),
                    ),
                    SizedBox(width: 10),
                    _entity!.hasBarcode
                        ? IconButton(
                            icon: Icon(Icons.qr_code), onPressed: _scanBarcode)
                        : Container()
                  ]),
          ),
      ),
    );
  }

  Future _addNewRow() async {
    var row = Map<String, dynamic>();
    row['i'] = widget.data.length;
    setState(() => widget.data.add(row));
  }
}
